!*************************************************************************
Module HamiltonianMod
!*************************************************************************
!	HDiagonalize calls LAPACK rountines

  use SysParams, 	only : double
  Use ConjGrad

  implicit none

  save

! hMaxSize	  Dimension of the space allocated for the hamiltonian
! hSize       Actual dimension of the hamiltonian
! h(i)        I-th element of the hamiltonian matrix in packed storage
! work arrays for LAPACK

  type hamiltonianArrayT
    integer                   :: hSize , hMaxSize, SqHSize
    complex(double), pointer  :: h(:)     
    complex(double), pointer  :: SqH(:,:)     

                                            
    complex(double), pointer  :: work(:)
    real(double),    pointer  :: rwork(:)
    integer,         pointer  :: iwork(:)
    integer,         pointer  :: fail(:)
  end type hamiltonianArrayT

  Type ParamsT
     integer          :: period,switch,precond,conv
     real(double)     :: tol
  end Type ParamsT 

! The module contains the public functions:
!	HInit
!	HDestroy
!	HDiagonalize (calls LAPACK rountines)
!	HPrint
! The module contains the public variables:
! The module contains the private functions:
! The module contains the private variables:
 
  contains

!*****************************************************************************
  Subroutine HInit(hamiltonian, maxHSize)
!*****************************************************************************
!   Allocate space for the hamiltonian and the work matrices.

    implicit none

    type( hamiltonianArrayT ), pointer :: hamiltonian
    integer, intent(in) :: maxHSize
    integer             :: hsize, error



    if( associated( hamiltonian )) stop 'Error, hamiltonian already allocated.'
    allocate( hamiltonian, stat = error )
    if( error /= 0 ) stop 'Error, hamiltonian allocation failed.'

    !!!!!!!!!!!!! CHANGED ddas !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!    hamiltonian%SqHSize = 180
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    hSize = 0
    hamiltonian%hMaxSize = maxHSize

    allocate( hamiltonian%h( hamiltonian%hMaxSize * ( hamiltonian%hMaxSize + 1 ) / 2 ), &
         !!!!!!!!CHANGED ddas !!!!!!!!!!!!!!!!!!!!!!!!!!!
         & hamiltonian%SqH( hamiltonian%hMaxSize,hamiltonian%hMaxSize), &   
         !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
         & hamiltonian%fail( hamiltonian%hMaxSize ), stat=error )
    allocate( hamiltonian%work( 2 * hamiltonian%hMaxSize ), &
         & hamiltonian%rwork( 7 * hamiltonian%hMaxSize ), &
         & hamiltonian%iwork( 5 *hamiltonian%hMaxSize ), stat=error )
    if(error /= 0) stop 'Error in allocating Hamiltonian space.'

  end Subroutine HInit

!*****************************************************************************
  Subroutine HDestroy( hamiltonian )
!*****************************************************************************

    implicit none

    type( hamiltonianArrayT ), pointer :: hamiltonian
    integer	:: error

    if( .not. associated( hamiltonian )) stop 'Error, hamiltonian not allocated.'

    deallocate( hamiltonian%h, hamiltonian%work, hamiltonian%rwork, &
         & hamiltonian%SqH, &
         & hamiltonian%iwork, hamiltonian%fail, stat=error )
    if(error /= 0) stop 'Error in deallocating Hamiltonian space.'

    deallocate( hamiltonian, stat = error )
    if(error /= 0) stop 'Error, hamiltonian deallocation failed.'

    nullify( hamiltonian )

  end Subroutine Hdestroy

!*****************************************************************************
  Subroutine HDiagonalize( hamiltonian, eigenStates, k , Params)
!*****************************************************************************
! Hamiltonian matrix is diagonalized by a call to LAPACK routines.

    use SysParams,      only : one, zero
    use EigenStatesMod, only : eigenStatesT

    implicit none

    type(hamiltonianArrayT), pointer :: hamiltonian
    type(eigenStatesT),      pointer :: eigenStates
    type(ParamsT)                    :: Params
    integer, intent(in)              :: k
    integer                          :: neig, error

    !!!!!!!!!!!!!!! CHANGED ddas !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    integer                                  :: r,s,HSize,NStates
    real(double)                             :: tol
    real(double), dimension(:), allocatable ::  EigVal 
    complex(double), dimension(:,:), allocatable :: EigState

    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    if(Params%Switch.eq.0) then
       call zhpevx( 'V','I','U', hamiltonian%hSize, hamiltonian%h, &
            & zero, zero, 1, eigenStates%numStates, -one, &
            & neig, eigenStates%eigenValues(:,k), &
            & eigenStates%eigenVectors(:,:,k), &
            & eigenStates%maxBasisVectors, &
            & hamiltonian%work, hamiltonian%rwork, &
            & hamiltonian%iwork, hamiltonian%fail(1:hamiltonian%hSize), &
            & error )

       if(error /= 0) stop 'Error: failed in zhpevx.'
    elseif(Params%Switch.eq.1) then

       allocate(EigVal(NStates), EigState(HSize,NStates))

       Call CGDiagonalise(hamiltonian%hSize,hamiltonian%SqH,eigenStates%numStates, &
            EigVal,EigState, Params%tol,Params%period,Params%precond,Params%conv)

       do r = 1, eigenStates%numStates 
          eigenStates%eigenValues(r,k) = EigVal(r)
          do s = 1, hamiltonian%hSize
             eigenStates%eigenVectors(s,r,k) = EigState(s,r)
          end do
       end do

       deallocate(EigVal,EigState)

    endif


  end Subroutine HDiagonalize


!*****************************************************************************
  Subroutine HPrint( hamiltonian )
!*****************************************************************************

    implicit none

    type( hamiltonianArrayT ), pointer :: hamiltonian
    integer :: i, j

    open(7, file = 'h.dat')

    do i = 1, hamiltonian%hSize
      do j = i, hamiltonian%hSize
        write(unit = 7, fmt = '(F8.4)', advance = 'NO') real(hamiltonian%h(i+j*(j-1)/2))
      end do
      write(unit = 7, fmt = '()')
    end do

  end Subroutine HPrint

end Module HamiltonianMod
