!****************************************************************************
Module Initialisation
!****************************************************************************

  Use TypeDef

  Implicit None



Contains

!****************************************************************************
  Subroutine MinVecAllocate(MinVec,NSize)
!****************************************************************************

    integer, intent(in) :: NSize
    type(MinVecT), pointer :: MinVec

    integer :: error

    if(associated(MinVec)) stop 'Error MinVec already allocated'

    allocate(MinVec, stat = error)

    if(error/=0) stop 'Error, Vector allocation failed.'

    allocate(MinVec%Phi(NSize),MinVec%StDesc(NSize), stat = error)

    if(error/=0) stop 'Error in allocating MinVec space'

  end Subroutine MinVecAllocate

!****************************************************************************
  Subroutine EigSystemAllocate(EigSystem,NSize,NEig)
!****************************************************************************
!
! This Subroutine allocates space for EigSystem components
!
    integer, intent(in) :: NSize,NEig
    type(EigSystemT), pointer :: EigSystem

    integer :: error

    if(associated(EigSystem)) stop 'Error EigSystem already allocated'

    allocate(EigSystem, stat = error) ! Allocate space for the type

    if(error/=0) stop 'Erorr, EigSystem allocation failed'

    allocate(EigSystem%States(NSize,NEig), EigSystem%EPsi(NSize), &
         & EigSystem%Values(NEig))

    if(error/=0) stop 'Error in allocating EigSystem space'

    ! Fill up the no. of eigenvalues sought
    EigSystem%HSize = NSize

  end Subroutine EigSystemAllocate

!****************************************************************************
  Subroutine EigSystemDestroy(EigSystem)
!***************************************************************************

    type(EigSystemT), pointer :: EigSystem

    deallocate(EigSystem%States,EigSystem%EPsi,EigSystem%Values)
    deallocate(EigSystem)


  end Subroutine EigSystemDestroy

!****************************************************************************
  Subroutine MinVecDestroy(MinVec)
!****************************************************************************

    type(MinVecT), pointer :: MinVec

    deallocate(MinVec%StDesc,MinVec%Phi)
    deallocate(MinVec)

  end Subroutine MinVecDestroy

!***************************************************************************
  Subroutine EigStateInit(EigSystem,iband)
!***************************************************************************

    integer, intent(in) :: iband
    type(EigSystemT),  pointer :: EigSystem

    integer :: NSize,r
    real(dbl) :: R1,rnd1,rnd2,del

    NSIze = EigSystem%HSize
    del = 0.1
    

    Call Random_Seed
    do r = 1, NSize
       Call Random_Number(R1)
       rnd1 = -1.d0 + 2.d0*R1
       Call Random_Number(R1)
       rnd2 = -1.d0 + 2.d0*R1
       EigSystem%States(r,iband) = cmplx(rnd1,rnd2)
    end do

    EigSystem%EPsi = cmplx(0.d0,0.d0)

  end Subroutine EigStateInit

!****************************************************************************
  Subroutine MinVecInit(MinVec,NSize)
!***************************************************************************

    integer, intent(in) :: NSize
    type(MinVecT), pointer :: MinVec

    ! Initialise all these vectors to 0.d0
    MinVec%Phi = cmplx(0.d0,0.d0)
    MinVec%StDesc = cmplx(0.d0,0.d0)

  end Subroutine MinVecInit

!***************************************************************************

end Module Initialisation

