!*************************************************************************
Program PWDensity
!*************************************************************************
! PW calculates the band structure along various directions in
! k-space using a plane-wave basis and a fixed "effective" potential.  
! Several choices of empirical pseudopotentials are provided.
! Written by William Mattson and Richard M. Martin of the University 
! of Illinois, based upon program written by K. Glassford and I. Souza.

! Note: atomic units (a.u.) are used throughout the program.

  use TagHandlerMod,       only : TagHandlerInit, TagHandlerDestroy, tagHandlerT
  use StructureMod,        only : StructureInit, StructureDestroy
  use KPointsMod,          only : MonkHorstPackinit, MonkHorstPackDestroy, &
                                & kPointsT, PlotBands
  use HamiltonianMod,      only : hamiltonianArrayT, HInit, HDestroy, & 
                                  & HDiagonalize, HPrint, ParamsT
  use EigenStatesMod,         only : EigenStatesInit, EigenStatesDestroy, &
                                & DensityMatrixInit, DensityMatrixPlot, &
                                & DensityMatrixDestroy
  use GVectorsMod,         only : gVectorsT, GVectorsInit, numGVectors, GVectorsDestroy, MaxGLen
  use SpecificHamMod,      only : PotentialInit, PotentialDestroy, &
  				& FindHMaxSize, hMaxSize, CalculateDensity &
                                & HamInfoT,HamInfoInit,HamInfoDestroy
  Use DensityArrayMod!,     only : DensityArrayT
			
  implicit none

  type(HamInfoT),          pointer :: HamInfo
  type(structureT),        pointer :: structure1
  type(kPointsT),          pointer :: kPoints
  type(tagHandlerT),       pointer :: tagHandler
  type(gVectorsT),         pointer :: gVectors
  type(eigenStatesT),      pointer :: eigenStates
  type(hamiltonianArrayT), pointer :: hamiltonian
  type(DensityMatrixT),    pointer :: MatrixOfDensity
  type(ParamsT)                    :: Params
  type(DensityArrayModT),  pointer :: DensityArray


  integer       :: kpt
  integer       :: HamArraySize
  character(80) :: filename

  print *, 'Input File Name'
  read(5,'(A80)') filename

  call TagHandlerInit( tagHandler, 10, filename) ! Initialize "tag handler" to read
                                    ! the input from file pw.dat
  call StructureInit( structure1, tagHandler )

  call GVectorsInit( gVectors, structure1, tagHandler )			! Gvector list and structurefactor

  call MonkHorstPackInit(kPoints,structure1%ndim,structure1%lattice,tagHandler)

  Call DensityArrayInit(DensityArray,kPoints%ndim,kPoints%ndiv)

  Call HamInfoInit(HamInfo,structure1,TagHandler)

  call PotentialInit(HamInfo,structure1)      	! Initialize potential in the PW Hamiltonian

  call FindHMaxSize( hamInfo, structure1, kPoints )	! Find hMaxSize = max size of H at any k point
  
  HamArraySize = HamInfo%HMaxSize

  call HInit(hamiltonian,HamArraySize)		! Maximum memory for H at any k point

!  call EigenStatesInit(numBands, numKPoints, hMaxSize)

  call EigenStatesInit( eigenStates, kPoints%numBands, kPoints%numKPoints, hamArraysize)

  eigenStates%eigenVectors = 0

  if(Params%Switch.eq.1) then
     open(unit=110,file='Report.cg',status='replace',action="write", & 
          & position="rewind")
     write(110,*) 'Eigenvalues:'
     close(unit=110)
  end if

! Creates files plot.dat and gnuplot.dat (Display with: gnuplot gnuplot.dat)
  do kpt = 1, kPoints%numKPoints                             ! loop over k-vectors

     call HGenerate( hamInfo, structure1, kPoints, eigenStates, &
          & kpt, hamiltonian%h, hamiltonian%SqH,hamiltonian%hSize )

     if(Params%Switch.eq.1) then
        open(unit=110,file='Report.cg',status='old',action="write", & 
             & position="append")
        write(110,*) ''
        write(110,*) 'k:',kpt
        close(unit=110)
     end if

      call HDiagonalize( hamiltonian, eigenStates, kpt , Params )

      print *, kpt,'/',kPoints%numKPoints,'completed'

    end do

!  call SolveHamiltonian(kPoints) ! Calculate the band structure

  call DensityMatrixInit(MatrixOfDensity,structure%ndim,kPoints%ndiv)  ! ddas: fill in something

  call CalculateDensity( hamInfo, structure1, kPoints, eigenStates, densityArray )
   
  call DensityMatrixDestroy(MatrixOfDensity)

  call EigenStatesDestroy(eigenStates)	!  Clean up memory

  call HDestroy(hamiltonian)
                
  call PotentialDestroy(haminfo)   	

  call MonkhorstPackDestroy( kPoints )

  call GVectorsDestroy( gVectors )

  call StructureDestroy( structure1 )

  call TagHandlerDestroy( tagHandler )     ! Clean up and close the input file

end Program PWDensity
